;===========================================================;
;                                                           ;
; Hangman                                                   ;
; Miscellaneous Routines                                    ;
;                                                           ;
;===========================================================;

;================================================
; invertScreen - Invert the contents of gbuf
;
; Input:    None
; Output:   None
;================================================
invertScreen:
#ifndef TI86
        ld      bc,768
#else
        ld      bc,1024
#endif                                          ; BC = Number of bytes of gbuf to invert
        ld      hl,gbuf
invertLoop:
        ld      a,(hl)
        cpl                                     ; Invert byte
        ld      (hl),a
        inc     hl
        dec     bc
        ld      a,b
        or      c
        jr      nz,invertLoop                   ; If BC != 0, loop
        ret

;================================================
; nextString - Move a data pointer to the next string in a data file
;
; Input:    HL => Current string
; Output:   HL => Next string
;================================================
nextString:
        ld      a,(hl)
        inc     hl
        or      a
        ret     z
        jr      nextString

;================================================
; getLetter - Check a key press to see if it corresponds to a letter
;
; Input:    None
; Output:   A = ASCII character, or 0 if no letter pressed
;================================================
getLetter:
        dec     a
        ld      e,a
        ld      d,0
        ld      hl,letterTable
        add     hl,de
        ld      a,(hl)
        sub     '.'
        ret     z
        add     a,'.'
        ret

;================================================
; saveVideo - Save gbuf to videoBuffer
;
; Input:    None
; Output:   None
;================================================
saveVideo:
        ld      hl,gbuf
        ld      de,videoBuffer
#ifdef TI86
        ld      bc,1024
#else
        ld      bc,768
#endif
        ldir
        ret

;================================================
; restoreVideo - Restore videoBuffer to gbuf
;
; Input:    None
; Output:   None
;================================================
restoreVideo:
        ld      hl,videoBuffer
        ld      de,gbuf
#ifdef TI86
        ld      bc,1024
#else
        ld      bc,768
#endif
        ldir
        ret

;================================================
; wait2nd - Wait for [2nd] to be pressed
;
; Input:    None
; Output:   None
;================================================
wait2nd:
        call    waitKey
        cp      GK_2ND
        jr      nz,wait2nd
        ret

;================================================
; waitKey - Wait for a key press
;
; Input:    None
; Output:   A = Key pressed (_getcsc style)
;================================================
waitKey:
        ei
wKeyLoop:
        halt
        bcall(GET_KEY)
        or      a
        jr      z,wKeyLoop
        ret

;================================================
; setShowHL - Show HL in small font at specified position
;
; Input:    HL = Value to show
;           DE = penrow, pencol
; Output:   None
;================================================
setShowHL:
        ld      (pencol),de

;================================================
; showHL - Show HL in small font at cursor position
;
; Input:    HL = Value to show
; Output:   None
;================================================
showHL:
        ld      de,stringBuffer+5
        ld      b,5
        xor     a
        ld      (de),a
showHLLoop:
        dec     de
        bcall(_divHLby10)
        add     a,'0'
        ld      (de),a
        djnz    showHLLoop
        ex      de,hl
        bcall(_vputs)
        ret

;================================================
; MirageOS routines that need to be written for Ion/TI-86 versions
;================================================
#ifdef MIRAGE

vPuts           = setVPuts                      ; Redirection to MirageOS library function

#else                                           ; Ion/TI-86 versions need their own copy of the function

;================================================
; directIn - Read keypresses from a key group
;
; Input:    A = Key group code
; Output:   A = Keypresses from specified group
;================================================
directIn:
        ld      b,a
        ld      a,$FF
        out     (1),a
        ld      a,b
        out     (1),a
        in      a,(1)
        ret

;================================================
; vPuts - Same as _vputs but sets cursor position
;
; Input:    DE = Value to store to (pencol)
;           HL => String to display
; Output:   HL => Next string
;================================================
vPuts:
        ld      (pencol),de
        bcall(_vputs)
        ret

#endif

#ifdef TI86                                     ; Routines not on the TI-86

;================================================
; ionFastCopyCenter - For TI-86 only, shift entire gbuf to the right 16 pixels then to ionFastCopy
;
; Input:    None
; Output:   None
;================================================
;ionFastCopyCenter:
;        ld      hl,gbuf+1021
;        ld      de,gbuf+1023
;        ld      bc,1022
;        lddr
;        inc     hl
;        ld      (hl),0
;        inc     hl
;        ld      (hl),0

;================================================
; ionFastCopy - Copy gbuf to lcd
;
; Input:    None
; Output:   None
;================================================
ionFastCopy:
        ld      hl,gbuf
        ld      de,videoMem
        ld      bc,1024
        ldir
        ret

;================================================
; random - Generate a random number
;
; Input:    B = Upper bound
; Output:   A = Random number, 0 <= A < B
;================================================
random:
        push    hl
        push    de
        ld      hl,(randData)
        ld      a,r
        ld      d,a
        ld      e,(hl)
        add     hl,de
        add     a,l
        xor     h
        ld      (randData),hl
        sbc     hl,hl
        ld      e,a
        ld      d,h
randomLoop:
        add     hl,de
        djnz    randomLoop
        ld      a,h
        pop     de
        pop     hl
        ret
randData        = _p_imathptr1

;================================================
; _grbufclr - Clear gbuf
;
; Input:    None
; Output:   None
;================================================
_grbufclr:
        ld      hl,gbuf
        ld      de,gbuf+1
        ld      bc,1023
        ld      (hl),0
        ldir
        ret

;================================================
; ionDetect - Find a program using a search string
;
; Authors:  Joe Wingbermuehle (http://joewing.calc.org)
;           Modified by James Vernon for TI-86
; Input:    HL => Place in VAT to start searching from
;           IX => Search string
; Output:   DE => Place stopped + 1
;           HL => Program data (after search string)
;           Z = Found
;================================================
ionDetect:
        call    _ram_page_7                     ; Load RAM page 7 (where VAT is stored)
        ld      de,(_ptemp_end+1)               ; DE => End of VAT
        call    _cphlde                         ; Are we at the end of the VAT?
        ld      a,(hl)                          ; A = Data type
        jr      nz,detectContinue               ; If not at the end of the VAT, check this entry
        call    _ram_page_1
        inc     a                               ; NZ
        ret                                     ; Leave
detectContinue:
        push    hl                              ; Save ptr to VAT entry
        cp      $0C                             ; Is it a string?
        jr      nz,detectSkip                   ; If not, try next entry
        dec     hl
        ld      e,(hl)
        dec     hl
        ld      d,(hl)
        dec     hl
        ld      a,(hl)
        ex      de,hl
        call    _ram_page_1
        ld      (realFilePtr),hl
        ld      (realFilePtr+2),a
        call    _set_abs_src_addr
;;        call    _get_word_ahl                   ; DE = Length of file
;;        ld      (__file_length),de              ; Save it to our code (see below 2 lines
        ld      a,1
        ld      hl,tempProgStorage-$8000
        call    _set_abs_dest_addr
        xor     a
        ld      hl,6000                         ; AHL = 6000 bytes to copy (more than enough to cover a compressed library file)
;;        ld      hl,$0000
;;__file_length                   = $-2
        call    _set_mm_num_bytes
        call    _mm_ldir
        call    _ram_page_1
        ld      hl,tempProgStorage+2
        push    ix
        pop     de
detectCheck:
        ld      a,(de)
        or      a
        jr      z,detectFound
        cp      (hl)
        inc     de
        inc     hl
        jr      z,detectCheck
detectNoGood:
        call    _ram_page_7
detectSkip:
        pop     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        ld      b,(hl)
        dec     hl
detectNameLoop2:
        dec     hl
        djnz    detectNameLoop2
        jr      ionDetect
detectFound:
        ex      (sp),hl
        call    _ram_page_7
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        ld      b,(hl)
        dec     hl
detectNameLoop1:
        dec     hl
        djnz    detectNameLoop1
        pop     de
        ex      de,hl
        call    _ram_page_1
        xor     a
        ret

#endif

.end
